@@ -12,8 +12,10 @@ package PDL::NiceSlice;
 #
 # Modified 2-Oct-2001: don't modify $var(LIST) if it's part of a
 # "for $var(LIST)" or "foreach $var(LIST)" statement.  CED.
+#
+# Modified 5-Nov-2007: stop processing if we encounter m/^no\s+PDL\;:\;:NiceSlice\;\s*$/.
 
-$PDL::NiceSlice::VERSION = '0.99.2';
+$PDL::NiceSlice::VERSION = '1.0.1';
 $PDL::NiceSlice::debug = 0;
 # the next one is largely stolen from Regexp::Common
 my $RE_cmt = qr'(?:(?:\#)(?:[^\n]*)(?:\n))';
@@ -29,6 +31,12 @@ eval << 'EOF' unless PDL->can('flat');
     }
 EOF
 
+##
+## As near as I can tell the code in this next eval is now vestigial -- 
+## it gets overwritten by the PDL::nslice definition in Core.pm. 
+##         -- CED 08-April-2004
+##
+
 eval << 'EOH';
   {
     package PDL;
@@ -36,46 +44,52 @@ eval << 'EOH';
 		    $_[0]->nelem == 1 ? $_[0]->flat->at(0) :
 		      die "multielement piddle where only one allowed" :
 			die "non piddle ref '".ref $_[0]."'"
-			  : int $_[0] }
+			: ($_[0] =~ s/^\*// ? "*".((int $_[0])||1) : int $_[0]); }
     sub PDL::nslice {
       my($pdl) = shift;
       my @args = @_;
       my ($i,$noslice) = (0,0);
+
       for (@args) {
 	if (UNIVERSAL::isa($_,'PDL')) {
 	  if ($_->nelem > 1) {
 	    if ($_->getndims > 1) {
-	      # allow one multi-D arg which will imply flat addressing
+	      ## allow one multi-D arg which will imply flat addressing
 	      PDL::Core::barf 'piddle must be <= 1D' if @args > 1;
 	      $pdl = $pdl->flat->index($_);
 	      $noslice = 1;
 	    } else {
-	      # dice this axis
+	      ## dice this axis
 	      $pdl = $pdl->dice_axis($i,$_);
-	      # and keep resulting dim fully in slice
+	      ## and keep resulting dim fully in slice
 	      $_ = 'X'; 
 	    }
-	  } else { $_ = $_->flat->at(0) } # reduce this one-element piddle
-					# to a scalar for 'slice'
+	  } else { $_ = $_->flat->at(0) } ## reduce this one-element piddle
+					## to a scalar for 'slice'
 	}
 	$i++;
       }
+
       return $pdl if $noslice;
-      # print STDERR 'processed arglist: ',join(',',@args);
+      ## print STDERR 'processed arglist: ',join(',',@args);
       my $slstr = join ',',(map {
 	!ref $_ && $_ eq "X" ? ":" :
 	  ref $_ eq "ARRAY" ? $#$_ > 1 && _intpar @$_[2] == 0 ? 
 	    "("._intpar(@$_[0]).")" : join ':', map {_intpar $_} @$_ :
 	      _intpar $_
 	    } @args);
-      # print STDERR "slicestr: $slstr\n";
+      print STDERR "slicestr: $slstr\n";
+      print "slicestr: '$slstr'!\n";
       return $pdl->slice($slstr);
     }
   }
 EOH
 
 # mark as lvalue sub if 5.6.x and above
-eval 'sub PDL::nslice : lvalue;' if ($^V and $^V ge v5.6.0);
+ eval {
+  no strict;
+  eval 'sub PDL::nslice : lvalue;' if ($^V and $^V > 5.006000);
+ }
 }
 
 use Text::Balanced; # used to find parenthesis-delimited blocks 
@@ -146,7 +160,7 @@ sub splitprotected ($$) {
 
 # a pattern that finds occurences of the form
 #
-#  $avar(
+#  $var(
 #
 # and
 #
@@ -183,6 +197,11 @@ sub onearg ($) {
   # we don't allow [] syntax (although that's what nslice internally uses)
   filterdie "invalid slice expression containing '[', expression was '".
     curarg()."'" if $arg =~ /^\s*\[/;
+
+  # If the arg starts with '*' it's a dummy call -- force stringification
+  # and prepend a '*' for handling by slice.
+  return "(q(*).($arg))" if($arg =~ s/^\*//);
+
   # this must be a simple position, leave as is
   return "$arg";
 }
@@ -216,7 +235,7 @@ sub findslice {
     print STDERR "pass $ct: found slice expr $found at line ".line()."\n"
       if $verb;
 
-#  Do final check for "for $avar(LIST)" and "foreach $avar(LIST)" syntax. 
+#  Do final check for "for $var(LIST)" and "foreach $var(LIST)" syntax. 
 #  Process into an 'nslice' call only if it's not that.
 
     if ($prefix =~ m/for(each)?(\s+(my|our))?\s+\$\w+(\s|$RE_cmt)*$/s ||
@@ -224,7 +243,10 @@ sub findslice {
 	$prefix =~ m/->\s*\$\w+$/s) # e.g. $a->$method(args)
       # method invocation via string, don't translate either
     {
-      $processed .= "$prefix".$found;
+	# note: even though we reject this one we need to call
+        #       findslice on $found in case
+	#       it contains slice expressions
+      $processed .= "$prefix".findslice($found);
     } else {      # statement is a real slice and not a foreach
 
       my ($call,$pre,$post,$arg);
@@ -305,12 +327,105 @@ sub findslice {
   $processed .= $ct > 0 ? substr $src, pos($src) : $src;
 }
 
+##############################
+# termstr - generate a regexp to find turn-me-off strings
+# CED 5-Nov-2007
+sub terminator_regexp{
+    my $clstr = shift;
+    $clstr =~ s/([^a-zA-Z0-9])/\\$1/g;
+    my $termstr = '^\s*no\s+'.$clstr.'\s*;\s*(#.*)*$';
+    return qr/$termstr/o; # allow trailing comments
+}
+
+sub reinstator_regexp{
+    my $clstr = shift;
+    $clstr =~ s/([^a-zA-Z0-9])/\\$1/g;
+    my $reinstr = '^\s*use\s+'.$clstr.'\s*;\s*(#.*)*$';
+    return qr/$reinstr/o; # allow trailing comments
+}
 
 # save eval of findslice that should be used within perldl as a preprocessor
 sub perldlpp {
- my ($txt) = @_;
+ my ($class, $txt) = @_;
+
+ ##############################
+ # Backwards compatibility to before the two-parameter form. The only
+ # call should be around line 206 of PDL::AutoLoader, but one never
+ # knows....
+ #    -- CED 5-Nov-2007
+ if(!defined($txt)) { 
+     print "PDL::NiceSlice::perldlpp -- got deprecated one-argument form, from ".(join("; ",caller))."...\n";
+     $txt = $class; 
+     $class = "PDL::NiceSlice";
+ }
+
+ ## Debugging to track exactly what is going on -- left in, in case it's needed again
+ if($PDL::debug > 1) {
+     print "PDL::NiceSlice::perldlpp - got:\n$txt\n";
+     my $i;
+     for $i(0..5){
+	 my($package,$filename,$line,$subroutine, $hasargs) = caller($i);
+	 printf("layer %d: %20s, %40s, line %5d, sub %20s, args: %d\n",$i,$package,$filename,$line,$subroutine,$hasargs);
+     }
+ }
+
  my $new;
- eval '$new = PDL::NiceSlice::findslice $txt';
+
+ ##############################
+ ## This block sort-of echoes import(), below...
+ ## Crucial difference: we don't give up the ghost on termination conditions, only
+ ## mask out current findslices.  That's because future uses won't be processed
+ ## (for some reason source filters don't work on evals).
+
+ my @lines= split /\n/,$txt;
+
+ my $terminator = terminator_regexp($class);
+ my $reinstator = reinstator_regexp($class);
+
+ my($status, $off, $end);
+ eval {
+     do {
+	 my $data = "";
+	 while(@lines) {
+	     $_= shift @lines;
+	     if(defined($terminator) && m/$terminator/) {
+		 $_ = "## $_";
+		 $off = 1;
+		 last;
+	     }
+	     if(defined($reinstator) && m/$reinstator/) {
+		 $_ = "## $_";
+	     }
+	     if(m/^\s*(__END__|__DATA__)\s*$/) {
+		 $end=$1; $off = 1;
+		 last;
+	     }
+	     $data .= "$_\n";
+	     $count++;
+	     $_="";
+	 }
+	 $_ = $data;
+	 $_ = findslice $_ ;
+	 $_ .= "no $class;\n" if $off;
+	 $_ .= "$end\n" if $end;
+	 $new .= "$_";
+	 
+	 while($off && @lines) {
+	     $_ = shift @lines;
+	     if(defined($reinstator) && m/$reinstator/) {
+		 $off = 0;
+		 $_ = "## $_";
+	     }
+	     if(defined($terminator) && m/$terminator/) {
+		 $_ = "## $_";
+	     }
+
+	     $new .= "$_\n";
+
+	 }
+     } while(@lines && !$end);
+ };
+     
  if ($@) {
    my $err = $@;
    for (split '','#!|\'"%~/') {
@@ -320,24 +435,39 @@ sub perldlpp {
    return "print q{NiceSlice error: $err}"; # if this doesn't work
                                                # we're stuffed
  }
+
+ if($PDL::debug > 1) {
+     print "PDL::NiceSlice::perldlpp - returning:\n$new\n";
+ }
  return $new;
 }
 
 use Filter::Util::Call;
 
+
+##############################
+# If you mess with the import filter, please also change the pdlpp importer 
+# just above this comment!  They both do similar things, but one to an eval string
+# and one to an import file.
+#   --CED 5-Nov-2007
+#
 sub import {
-  my ($class) = @_;
-  ($file,$offset) = (caller)[1,2];  # for error reporting
-  $offset++;
-  my $terminator = qr/^\s*no\s+$class\s*;\s*(#.*)*$/; # allow trailing comments
-	filter_add(
+    my ($class) = @_;
+    ($file,$offset) = (caller)[1,2];  # for error reporting
+    $offset++;
+    
+    ## Parse class name into a regexp suitable for filtration
+    my $terminator = terminator_regexp($class);
+
+    filter_add(
 		sub {
-			my ($status, $off, $end);
-			my $count = 0;
-			my $data = "";
+		    my ($status, $off, $end);
+		    my $count = 0;
+		    my $data = "";
 			while ($status = filter_read()) {
 				return $status if $status < 0;
-				if ($terminator && m/$terminator/) {
+		
+				if (defined($terminator) && m/$terminator/) {
 					$off=1;
 					last;
 				}
@@ -380,6 +510,8 @@ PDL::NiceSlice - toward a nicer slicing syntax for PDL
   $a(myfunc(0,$var),1:4)++; # when using functions in slice expressions
                             # use parentheses around args!
 
+  $b = $a(*3);              # Add dummy dimension of order 3
+
   # modifiers are specified in a ;-separated trailing block
   $a($a!=3;?)++;            # short for $a->where($a!=3)++
   $a(0:1114;_) .= 0;        # short for $a->flat->(0:1114)
@@ -401,13 +533,13 @@ Slicing is a basic, extremely common operation, and PDL's
 L<slice|PDL::Slices/slice> method would be cumbersome to use in many
 cases.  C<PDL::NiceSlice> rectifies that by incorporating new slicing
 syntax directly into the language via a perl I<source filter> (see
-L<perlfilter|the perlfilter man page>).  NiceSlice adds no new functionality, only convenient
-syntax.
+L<the perlfilter man page|perlfilter>).  NiceSlice adds no new functionality, only convenient syntax.
 
 NiceSlice is loaded automatically in the perldl shell, but (to avoid
 conflicts with other modules) must be loaded automatically in standalone
 perl/PDL scripts (see below).  If you prefer not to use a prefilter on
-your standalone scripts, you can use the L<slice> method in those scripts,
+your standalone scripts, you can use the L<slice|PDL::Slices/slice>
+method in those scripts,
 rather than the more compact NiceSlice constructs.
 
 =head1 Use in scripts and C<perldl> shell
@@ -422,7 +554,7 @@ how to enable the new slicing syntax within older L<perldl|perldl>.
 
 But now back to scripts and modules.
 Everything after C<use PDL::NiceSlice> will be translated
-and you can use the snew slicing syntax. Source filtering
+and you can use the new slicing syntax. Source filtering
 will continue until the end of the file is encountered.
 You can stop sourcefiltering before the end of the file
 by issuing a C<no PDL::NiceSlice> statement.
@@ -617,9 +749,9 @@ interfering with the current subref syntax, it will be treated as an
 invocation of the code reference C<$a> with argumentlist C<(4,5)>.
 
 The $a(ARGS) syntax collides in a minor way with the perl syntax.  In
-particular, ``foreach $avar(LIST)'' appears like a PDL slicing call.  
-NiceSlice avoids translating the ``for $avar(LIST)'' and 
-``foreach $avar(LIST)'' constructs for this reason.  Since you
+particular, ``foreach $var(LIST)'' appears like a PDL slicing call.  
+NiceSlice avoids translating the ``for $var(LIST)'' and 
+``foreach $var(LIST)'' constructs for this reason.  Since you
 can't use just any old lvalue expression in the 'foreach' 'for'
 constructs -- only a real perl scalar will do -- there's no 
 functionality lost.  If later versions of perl accept 
@@ -910,6 +1042,12 @@ these dimensions will be fully kept in the sliced piddle:
   print $a((0),,)->info;    # similar
  PDL: Double D [4,5]
 
+=item * dummy dimensions
+
+As in L<slice|slice>, you can insert a dummy dimension by preceding a
+single index argument with '*'.  A lone '*' inserts a dummy dimension of 
+order 1; a '*' followed by a number inserts a dummy dimension of that order.
+
 =item * piddle index lists
 
 The second way to select indices from a dimension is via 1D piddles
@@ -1072,7 +1210,7 @@ E<lt>pdl-porters@jach.hawaii.eduE<gt>.
 Copyright (c) 2001, 2002 Christian Soeller. All Rights Reserved.
 This module is free software. It may be used, redistributed
 and/or modified under the same terms as PDL itself
-(see http://pdl.perl.org).
+(see L<http://pdl.perl.org>).
 
 =cut
 
