@@ -167,6 +167,8 @@ sub new {
       OpenGL::glutInitDisplayMode( OpenGL::GLUT_RGBA() | OpenGL::GLUT_DOUBLE() | OpenGL::GLUT_DEPTH() );        # hardwire for now
 
       my($glutwin) = OpenGL::glutCreateWindow( "GLUT TriD" );
+      OpenGL::glutSetWindowTitle("GLUT TriD #$glutwin");        # add GLUT window id to title
+
       $self = { 'glutwindow' => $glutwin, 'xevents' => \@fakeXEvents, 'winobjects' => \@winObjects };
 
       OpenGL::glutReshapeFunc( \&_pdl_fake_ConfigureNotify );
@@ -176,7 +178,7 @@ sub new {
       OpenGL::glutMotionFunc( \&_pdl_fake_MotionNotify );
       OpenGL::glutDisplayFunc( \&_pdl_display_wrapper );
 
-      OpenGL::glutSetOption(OpenGL::GLUT_ACTION_ON_WINDOW_CLOSE, OpenGL::GLUT_ACTION_GLUTMAINLOOP_RETURNS) if OpenGL::_have_freeglut();
+      OpenGL::glutSetOption(OpenGL::GLUT_ACTION_ON_WINDOW_CLOSE(), OpenGL::GLUT_ACTION_GLUTMAINLOOP_RETURNS()) if OpenGL::_have_freeglut();
 
       OpenGL::glutMainLoopEvent();       # pump event loop so window appears
    }
